#****************************************************************************
#**
#**  Author(s):  Exavier Macbeth
#**
#**  Summary  :  BlackOps: Adv Command Unit - Serephim ACU
#**
#**  Copyright  2005 Gas Powered Games, Inc.  All rights reserved.
#****************************************************************************

local SWalkingLandUnit = import('/lua/seraphimunits.lua').SWalkingLandUnit
local Buff = import('/lua/sim/Buff.lua')

local SWeapons = import('/lua/seraphimweapons.lua')
local SDFChronotronCannonWeapon = SWeapons.SDFChronotronCannonWeapon
local SDFChronotronOverChargeCannonWeapon = SWeapons.SDFChronotronCannonOverChargeWeapon
local SIFCommanderDeathWeapon = SWeapons.SIFCommanderDeathWeapon
local EffectTemplate = import('/lua/EffectTemplates.lua')
local EffectUtil = import('/lua/EffectUtilities.lua')
local SIFLaanseTacticalMissileLauncher = SWeapons.SIFLaanseTacticalMissileLauncher
local AIUtils = import('/lua/ai/aiutilities.lua')
local SDFAireauWeapon = SWeapons.SDFAireauWeapon
local SDFSinnuntheWeapon = SWeapons.SDFSinnuntheWeapon
local SANUallCavitationTorpedo = SWeapons.SANUallCavitationTorpedo
local SeraACURapidWeapon = import('/mods/BlackOpsACUs/lua/EXBlackOpsweapons.lua').SeraACURapidWeapon 
local SeraACUBigBallWeapon = import('/mods/BlackOpsACUs/lua/EXBlackOpsweapons.lua').SeraACUBigBallWeapon 
local SAAOlarisCannonWeapon = SWeapons.SAAOlarisCannonWeapon

# Setup as RemoteViewing child unit rather than SWalkingLandUnit
local RemoteViewing = import('/lua/RemoteViewing.lua').RemoteViewing
SWalkingLandUnit = RemoteViewing( SWalkingLandUnit ) 

ESL0001 = Class( SWalkingLandUnit ) {
    DeathThreadDestructionWaitTime = 2,

    Weapons = {
        DeathWeapon = Class(SIFCommanderDeathWeapon) {},
        ChronotronCannon = Class(SDFChronotronCannonWeapon) {},
        EXTorpedoLauncher01 = Class(SANUallCavitationTorpedo) {},
        EXTorpedoLauncher02 = Class(SANUallCavitationTorpedo) {},
        EXTorpedoLauncher03 = Class(SANUallCavitationTorpedo) {},
        EXBigBallCannon01 = Class(SeraACUBigBallWeapon) {
            PlayFxMuzzleChargeSequence = function(self, muzzle)
                #CreateRotator(unit, bone, axis, [goal], [speed], [accel], [goalspeed])
                if not self.ClawTopRotator then 
                    self.ClawTopRotator = CreateRotator(self.unit, 'Pincer_Upper', 'x')
                    self.ClawBottomRotator = CreateRotator(self.unit, 'Pincer_Lower', 'x')
                    
                    self.unit.Trash:Add(self.ClawTopRotator)
                    self.unit.Trash:Add(self.ClawBottomRotator)
                end
                
                self.ClawTopRotator:SetGoal(-15):SetSpeed(10)
                self.ClawBottomRotator:SetGoal(15):SetSpeed(10)
                
                SDFSinnuntheWeapon.PlayFxMuzzleChargeSequence(self, muzzle)
                
                self:ForkThread(function()
                    WaitSeconds(self.unit:GetBlueprint().Weapon[7].MuzzleChargeDelay)
                    
                    self.ClawTopRotator:SetGoal(0):SetSpeed(50)
                    self.ClawBottomRotator:SetGoal(0):SetSpeed(50)
                end)
            end,
		},
        EXBigBallCannon02 = Class(SeraACUBigBallWeapon) {
            PlayFxMuzzleChargeSequence = function(self, muzzle)
                #CreateRotator(unit, bone, axis, [goal], [speed], [accel], [goalspeed])
                if not self.ClawTopRotator then 
                    self.ClawTopRotator = CreateRotator(self.unit, 'Pincer_Upper', 'x')
                    self.ClawBottomRotator = CreateRotator(self.unit, 'Pincer_Lower', 'x')
                    
                    self.unit.Trash:Add(self.ClawTopRotator)
                    self.unit.Trash:Add(self.ClawBottomRotator)
                end
                
                self.ClawTopRotator:SetGoal(-15):SetSpeed(10)
                self.ClawBottomRotator:SetGoal(15):SetSpeed(10)
                
                SDFSinnuntheWeapon.PlayFxMuzzleChargeSequence(self, muzzle)
                
                self:ForkThread(function()
                    WaitSeconds(self.unit:GetBlueprint().Weapon[8].MuzzleChargeDelay)
                    
                    self.ClawTopRotator:SetGoal(0):SetSpeed(50)
                    self.ClawBottomRotator:SetGoal(0):SetSpeed(50)
                end)
            end,
		},
        EXBigBallCannon03 = Class(SeraACUBigBallWeapon) {
            PlayFxMuzzleChargeSequence = function(self, muzzle)
                #CreateRotator(unit, bone, axis, [goal], [speed], [accel], [goalspeed])
                if not self.ClawTopRotator then 
                    self.ClawTopRotator = CreateRotator(self.unit, 'Pincer_Upper', 'x')
                    self.ClawBottomRotator = CreateRotator(self.unit, 'Pincer_Lower', 'x')
                    
                    self.unit.Trash:Add(self.ClawTopRotator)
                    self.unit.Trash:Add(self.ClawBottomRotator)
                end
                
                self.ClawTopRotator:SetGoal(-15):SetSpeed(10)
                self.ClawBottomRotator:SetGoal(15):SetSpeed(10)
                
                SDFSinnuntheWeapon.PlayFxMuzzleChargeSequence(self, muzzle)
                
                self:ForkThread(function()
                    WaitSeconds(self.unit:GetBlueprint().Weapon[9].MuzzleChargeDelay)
                    
                    self.ClawTopRotator:SetGoal(0):SetSpeed(50)
                    self.ClawBottomRotator:SetGoal(0):SetSpeed(50)
                end)
            end,
		},
        EXRapidCannon01 = Class(SeraACURapidWeapon) {},
        EXRapidCannon02 = Class(SeraACURapidWeapon) {},
        EXRapidCannon03 = Class(SeraACURapidWeapon) {},
        EXAA01 = Class(SAAOlarisCannonWeapon) {},
        EXAA02 = Class(SAAOlarisCannonWeapon) {},
        Missile = Class(SIFLaanseTacticalMissileLauncher) {
            CurrentRack = 1,
                
            PlayFxMuzzleSequence = function(self, muzzle)
                local bp = self:GetBlueprint()
                self.MissileRotator = CreateRotator(self.unit, bp.RackBones[self.CurrentRack].RackBone, 'x', nil, 0, 0, 0)
                muzzle = bp.RackBones[self.CurrentRack].MuzzleBones[1]
                self.MissileRotator:SetGoal(-10):SetSpeed(10)
                SIFLaanseTacticalMissileLauncher.PlayFxMuzzleSequence(self, muzzle)
                WaitFor(self.MissileRotator)
                WaitTicks(1)
            end,
                
            CreateProjectileAtMuzzle = function(self, muzzle)
                muzzle = self:GetBlueprint().RackBones[self.CurrentRack].MuzzleBones[1]
                if self.CurrentRack >= 2 then
                    self.CurrentRack = 1
                else
                    self.CurrentRack = self.CurrentRack + 1
                end
                SIFLaanseTacticalMissileLauncher.CreateProjectileAtMuzzle(self, muzzle)
            end,
                
            PlayFxRackReloadSequence = function(self)
                WaitTicks(1)
                self.MissileRotator:SetGoal(0):SetSpeed(10)
                WaitFor(self.MissileRotator)
                self.MissileRotator:Destroy()
                self.MissileRotator = nil
            end,
        },
        OverCharge = Class(SDFChronotronOverChargeCannonWeapon) {

            OnCreate = function(self)
                SDFChronotronOverChargeCannonWeapon.OnCreate(self)
                self:SetWeaponEnabled(false)
                self.AimControl:SetEnabled(false)
                self.AimControl:SetPrecedence(0)
				self.unit:SetOverchargePaused(false)
            end,

            OnEnableWeapon = function(self)
                if self:BeenDestroyed() then return end
                SDFChronotronOverChargeCannonWeapon.OnEnableWeapon(self)
                self:SetWeaponEnabled(true)
                self.unit:SetWeaponEnabledByLabel('ChronotronCannon', false)
                self.unit:BuildManipulatorSetEnabled(false)
                self.AimControl:SetEnabled(true)
                self.AimControl:SetPrecedence(20)
                self.unit.BuildArmManipulator:SetPrecedence(0)
                self.AimControl:SetHeadingPitch( self.unit:GetWeaponManipulatorByLabel('ChronotronCannon'):GetHeadingPitch() )
            end,
            
            OnWeaponFired = function(self)
                SDFChronotronOverChargeCannonWeapon.OnWeaponFired(self)
                self:OnDisableWeapon()
                self:ForkThread(self.PauseOvercharge)
            end,
            
            OnDisableWeapon = function(self)
                if self.unit:BeenDestroyed() then return end
                self:SetWeaponEnabled(false)
                self.unit:SetWeaponEnabledByLabel('ChronotronCannon', true)
                self.unit:BuildManipulatorSetEnabled(false)
                self.AimControl:SetEnabled(false)
                self.AimControl:SetPrecedence(0)
                self.unit.BuildArmManipulator:SetPrecedence(0)
                self.unit:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.AimControl:GetHeadingPitch() )
            end,
            
            PauseOvercharge = function(self)
                if not self.unit:IsOverchargePaused() then
                    self.unit:SetOverchargePaused(true)
                    WaitSeconds(1/self:GetBlueprint().RateOfFire)
                    self.unit:SetOverchargePaused(false)
                end
            end,
            
            OnFire = function(self)
                if not self.unit:IsOverchargePaused() then
                    SDFChronotronOverChargeCannonWeapon.OnFire(self)
                end
            end,
            IdleState = State(SDFChronotronOverChargeCannonWeapon.IdleState) {
                OnGotTarget = function(self)
                    if not self.unit:IsOverchargePaused() then
                        SDFChronotronOverChargeCannonWeapon.IdleState.OnGotTarget(self)
                    end
                end,            
                OnFire = function(self)
                    if not self.unit:IsOverchargePaused() then
                        ChangeState(self, self.RackSalvoFiringState)
                    end
                end,
            },
            RackSalvoFireReadyState = State(SDFChronotronOverChargeCannonWeapon.RackSalvoFireReadyState) {
                OnFire = function(self)
                    if not self.unit:IsOverchargePaused() then
                        SDFChronotronOverChargeCannonWeapon.RackSalvoFireReadyState.OnFire(self)
                    end
                end,
            },  
        },
    },


    OnCreate = function(self)
        SWalkingLandUnit.OnCreate(self)
        self:SetCapturable(false)
        self:SetupBuildBones()
        # Restrict what enhancements will enable later
        self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
        self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
    end,

    OnPrepareArmToBuild = function(self)
        SWalkingLandUnit.OnPrepareArmToBuild(self)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(true)
        self.BuildArmManipulator:SetPrecedence(20)
        self.wcBuildMode = true
		self:ForkThread(self.WeaponConfigCheck)
        self.BuildArmManipulator:SetHeadingPitch( self:GetWeaponManipulatorByLabel('ChronotronCannon'):GetHeadingPitch() )
    end,

    OnStopCapture = function(self, target)
        SWalkingLandUnit.OnStopCapture(self, target)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(false)
        self.BuildArmManipulator:SetPrecedence(0)
        self.wcBuildMode = false
		self:ForkThread(self.WeaponConfigCheck)
        self:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.BuildArmManipulator:GetHeadingPitch() )
    end,

    OnFailedCapture = function(self, target)
        SWalkingLandUnit.OnFailedCapture(self, target)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(false)
        self.BuildArmManipulator:SetPrecedence(0)
        self.wcBuildMode = false
		self:ForkThread(self.WeaponConfigCheck)
        self:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.BuildArmManipulator:GetHeadingPitch() )
    end,

    OnStopReclaim = function(self, target)
        SWalkingLandUnit.OnStopReclaim(self, target)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(false)
        self.BuildArmManipulator:SetPrecedence(0)
        self.wcBuildMode = false
		self:ForkThread(self.WeaponConfigCheck)
        self:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.BuildArmManipulator:GetHeadingPitch() )
    end,

    OnStopBeingBuilt = function(self,builder,layer)
        SWalkingLandUnit.OnStopBeingBuilt(self,builder,layer)
        self:DisableRemoteViewingButtons()
        self:ForkThread(self.GiveInitialResources)
        self.ShieldEffectsBag = {}
        self:DisableUnitIntel('RadarStealth')
        self:DisableUnitIntel('SonarStealth')
        self:DisableUnitIntel('Cloak')
        self:DisableUnitIntel('CloakField')
		self:HideBone('Engineering', true)
		self:HideBone('Combat_Engineering', true)
		self:HideBone('Rapid_Cannon', true)
		self:HideBone('Basic_Gun_Up', true)
		self:HideBone('Big_Ball_Cannon', true)
		self:HideBone('Torpedo_Launcher', true)
		self:HideBone('Missile_Launcher', true)
		self:HideBone('IntelPack', true)
		self:HideBone('L_Spinner_B01', true)
		self:HideBone('L_Spinner_B02', true)
		self:HideBone('L_Spinner_B03', true)
		self:HideBone('S_Spinner_B01', true)
		self:HideBone('S_Spinner_B02', true)
		self:HideBone('S_Spinner_B03', true)
		self:HideBone('Left_AA_Mount', true)
		self:HideBone('Right_AA_Mount', true)
		self.lambdaEmitterTable = {}
		if not self.RotatorManipulator1 then
            self.RotatorManipulator1 = CreateRotator( self, 'S_Spinner_B01', 'y' )
            self.Trash:Add( self.RotatorManipulator1 )
        end
        self.RotatorManipulator1:SetAccel( 30 )
        self.RotatorManipulator1:SetTargetSpeed( 120 )
		if not self.RotatorManipulator2 then
            self.RotatorManipulator2 = CreateRotator( self, 'L_Spinner_B01', 'y' )
            self.Trash:Add( self.RotatorManipulator2 )
        end
        self.RotatorManipulator2:SetAccel( -15 )
        self.RotatorManipulator2:SetTargetSpeed( -60 )
		self.wcBuildMode = false
		self.wcOCMode = false
		self.wcTorp01 = false
		self.wcTorp02 = false
		self.wcTorp03 = false
		self.wcBigBall01 = false
		self.wcBigBall02 = false
		self.wcBigBall03 = false
		self.wcRapid01 = false
		self.wcRapid02 = false
		self.wcRapid03 = false
		self.wcAA01 = false
		self.wcAA02 = false
		self.wcTMissiles01 = false
		self:ForkThread(self.WeaponRangeReset)
		self:ForkThread(self.WeaponConfigCheck)
    end,

    OnFailedToBuild = function(self)
        SWalkingLandUnit.OnFailedToBuild(self)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(false)
        self.BuildArmManipulator:SetPrecedence(0)
        self.wcBuildMode = false
		self:ForkThread(self.WeaponConfigCheck)
        self:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.BuildArmManipulator:GetHeadingPitch() )
    end,

    OnStartBuild = function(self, unitBeingBuilt, order)
        local bp = self:GetBlueprint()
        if order != 'Upgrade' or bp.Display.ShowBuildEffectsDuringUpgrade then
            self:StartBuildingEffects(unitBeingBuilt, order)
        end
        self:DoOnStartBuildCallbacks(unitBeingBuilt)
        self:SetActiveConsumptionActive()
        self:PlayUnitSound('Construct')
        self:PlayUnitAmbientSound('ConstructLoop')
        if bp.General.UpgradesTo and unitBeingBuilt:GetUnitId() == bp.General.UpgradesTo and order == 'Upgrade' then
            self.Upgrading = true
            self.BuildingUnit = false        
            unitBeingBuilt.DisallowCollisions = true
        end
        self.UnitBeingBuilt = unitBeingBuilt
        self.UnitBuildOrder = order
        self.BuildingUnit = true
    end,  

    OnStopBuild = function(self, unitBeingBuilt)
        SWalkingLandUnit.OnStopBuild(self, unitBeingBuilt)
        if self:BeenDestroyed() then return end
        self:BuildManipulatorSetEnabled(false)
        self.BuildArmManipulator:SetPrecedence(0)
        self.wcBuildMode = false
		self:ForkThread(self.WeaponConfigCheck)
        self:GetWeaponManipulatorByLabel('ChronotronCannon'):SetHeadingPitch( self.BuildArmManipulator:GetHeadingPitch() )
        self.UnitBeingBuilt = nil
        self.UnitBuildOrder = nil
        self.BuildingUnit = false
    end,
    
    PlayCommanderWarpInEffect = function(self)
        self:HideBone(0, true)
        self:SetUnSelectable(true)
        self:SetBusy(true)
        self:SetBlockCommandQueue(true)
        self:ForkThread(self.WarpInEffectThread)
    end, 
    
    WarpInEffectThread = function(self)
        self:PlayUnitSound('CommanderArrival')
        self:CreateProjectile( '/effects/entities/UnitTeleport01/UnitTeleport01_proj.bp', 0, 1.35, 0, nil, nil, nil):SetCollision(false)
        WaitSeconds(2.1)
        self:ShowBone(0, true)
        self:SetUnSelectable(false)
        self:SetBusy(false)
        self:SetBlockCommandQueue(false)
		self:HideBone('Engineering', true)
		self:HideBone('Combat_Engineering', true)
		self:HideBone('Rapid_Cannon', true)
		self:HideBone('Basic_Gun_Up', true)
		self:HideBone('Big_Ball_Cannon', true)
		self:HideBone('Torpedo_Launcher', true)
		self:HideBone('Missile_Launcher', true)
		self:HideBone('IntelPack', true)
		self:HideBone('L_Spinner_B01', true)
		self:HideBone('L_Spinner_B02', true)
		self:HideBone('L_Spinner_B03', true)
		self:HideBone('S_Spinner_B01', true)
		self:HideBone('S_Spinner_B02', true)
		self:HideBone('S_Spinner_B03', true)
		self:HideBone('Left_AA_Mount', true)
		self:HideBone('Right_AA_Mount', true)
        local totalBones = self:GetBoneCount() - 1
        local army = self:GetArmy()
        for k, v in EffectTemplate.UnitTeleportSteam01 do
            for bone = 1, totalBones do
                CreateAttachedEmitter(self,bone,army, v)
            end
        end

        WaitSeconds(6)
    end,

    GiveInitialResources = function(self)
        WaitTicks(2)
        self:GetAIBrain():GiveResource('Energy', self:GetBlueprint().Economy.StorageEnergy)
        self:GetAIBrain():GiveResource('Mass', self:GetBlueprint().Economy.StorageMass)
    end,

    CreateBuildEffects = function( self, unitBeingBuilt, order )
        EffectUtil.CreateSeraphimUnitEngineerBuildingEffects( self, unitBeingBuilt, self:GetBlueprint().General.BuildBones.BuildEffectBones, self.BuildEffectsBag )
    end,

    WeaponRangeReset = function(self)
		if not self.wcTorp01 then
			local wepTorpedo01 = self:GetWeaponByLabel('EXTorpedoLauncher01')
			wepTorpedo01:ChangeMaxRadius(1)
		end
		if not self.wcTorp02 then
			local wepTorpedo02 = self:GetWeaponByLabel('EXTorpedoLauncher02')
			wepTorpedo02:ChangeMaxRadius(1)
		end
		if not self.wcTorp03 then
			local wepTorpedo03 = self:GetWeaponByLabel('EXTorpedoLauncher03')
			wepTorpedo03:ChangeMaxRadius(1)
		end
		if not self.wcBigBall01 then
			local wepAntiMatter01 = self:GetWeaponByLabel('EXBigBallCannon01')
			wepAntiMatter01:ChangeMaxRadius(1)
		end
		if not self.wcBigBall02 then
			local wepAntiMatter02 = self:GetWeaponByLabel('EXBigBallCannon02')
			wepAntiMatter02:ChangeMaxRadius(1)
		end
		if not self.wcBigBall03 then
			local wepAntiMatter03 = self:GetWeaponByLabel('EXBigBallCannon03')
			wepAntiMatter03:ChangeMaxRadius(1)
		end
		if not self.wcRapid01 then
			local wepGattling01 = self:GetWeaponByLabel('EXRapidCannon01')
			wepGattling01:ChangeMaxRadius(1)
		end
		if not self.wcRapid02 then
			local wepGattling02 = self:GetWeaponByLabel('EXRapidCannon02')
			wepGattling02:ChangeMaxRadius(1)
		end
		if not self.wcRapid03 then
			local wepGattling03 = self:GetWeaponByLabel('EXRapidCannon03')
			wepGattling03:ChangeMaxRadius(1)
		end
		if not self.wcAA01 then
			local wepLance01 = self:GetWeaponByLabel('EXAA01')
			wepLance01:ChangeMaxRadius(1)
		end
		if not self.wcAA02 then
			local wepLance02 = self:GetWeaponByLabel('EXAA02')
			wepLance02:ChangeMaxRadius(1)
		end
		if not self.wcTMissiles01 then
			local wepTacMiss = self:GetWeaponByLabel('Missile')
			wepTacMiss:ChangeMaxRadius(1)
		end
    end,
	
    WeaponConfigCheck = function(self)
		if self.wcBuildMode then
			self:SetWeaponEnabledByLabel('ChronotronCannon', false)
			self:SetWeaponEnabledByLabel('OverCharge', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher01', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher02', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher03', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon01', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon02', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon03', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon01', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon02', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon03', false)
			self:SetWeaponEnabledByLabel('EXAA01', false)
			self:SetWeaponEnabledByLabel('EXAA02', false)
			self:SetWeaponEnabledByLabel('Missile', false)
		end
		if self.wcOCMode then
			self:SetWeaponEnabledByLabel('ChronotronCannon', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher01', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher02', false)
			self:SetWeaponEnabledByLabel('EXTorpedoLauncher03', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon01', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon02', false)
			self:SetWeaponEnabledByLabel('EXBigBallCannon03', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon01', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon02', false)
			self:SetWeaponEnabledByLabel('EXRapidCannon03', false)
			self:SetWeaponEnabledByLabel('EXAA01', false)
			self:SetWeaponEnabledByLabel('EXAA02', false)
		end
		if not self.wcBuildMode and not self.wcOCMode then
			self:SetWeaponEnabledByLabel('ChronotronCannon', true)
			self:SetWeaponEnabledByLabel('OverCharge', false)
			if self.wcTorp01 then
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher01', true)
				local wepTorpedo01 = self:GetWeaponByLabel('EXTorpedoLauncher01')
				wepTorpedo01:ChangeMaxRadius(60)
			else
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher01', false)
			end
			if self.wcTorp02 then
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher02', true)
				local wepTorpedo02 = self:GetWeaponByLabel('EXTorpedoLauncher02')
				wepTorpedo02:ChangeMaxRadius(60)
			else
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher02', false)
			end
			if self.wcTorp03 then
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher03', true)
				local wepTorpedo03 = self:GetWeaponByLabel('EXTorpedoLauncher03')
				wepTorpedo03:ChangeMaxRadius(60)
			else
				self:SetWeaponEnabledByLabel('EXTorpedoLauncher03', false)
			end
			if self.wcBigBall01 then
				self:SetWeaponEnabledByLabel('EXBigBallCannon01', true)
				local wepAntiMatter01 = self:GetWeaponByLabel('EXBigBallCannon01')
				wepAntiMatter01:ChangeMaxRadius(35)
			else
				self:SetWeaponEnabledByLabel('EXBigBallCannon01', false)
			end
			if self.wcBigBall02 then
				self:SetWeaponEnabledByLabel('EXBigBallCannon02', true)
				local wepAntiMatter02 = self:GetWeaponByLabel('EXBigBallCannon02')
				wepAntiMatter02:ChangeMaxRadius(45)
			else
				self:SetWeaponEnabledByLabel('EXBigBallCannon02', false)
			end
			if self.wcBigBall03 then
				self:SetWeaponEnabledByLabel('EXBigBallCannon03', true)
				local wepAntiMatter03 = self:GetWeaponByLabel('EXBigBallCannon03')
				wepAntiMatter03:ChangeMaxRadius(45)
			else
				self:SetWeaponEnabledByLabel('EXBigBallCannon03', false)
			end
			if self.wcRapid01 then
				self:SetWeaponEnabledByLabel('EXRapidCannon01', true)
				local wepGattling01 = self:GetWeaponByLabel('EXRapidCannon01')
				wepGattling01:ChangeMaxRadius(35)
			else
				self:SetWeaponEnabledByLabel('EXRapidCannon01', false)
			end
			if self.wcRapid02 then
				self:SetWeaponEnabledByLabel('EXRapidCannon02', true)
				local wepGattling02 = self:GetWeaponByLabel('EXRapidCannon02')
				wepGattling02:ChangeMaxRadius(35)
			else
				self:SetWeaponEnabledByLabel('EXRapidCannon02', false)
			end
			if self.wcRapid03 then
				self:SetWeaponEnabledByLabel('EXRapidCannon03', true)
				local wepGattling03 = self:GetWeaponByLabel('EXRapidCannon03')
				wepGattling03:ChangeMaxRadius(35)
			else
				self:SetWeaponEnabledByLabel('EXRapidCannon03', false)
			end
			if self.wcAA01 then
				self:SetWeaponEnabledByLabel('EXAA01', true)
				local wepLance01 = self:GetWeaponByLabel('EXAA01')
				wepLance01:ChangeMaxRadius(22)
			else
				self:SetWeaponEnabledByLabel('EXAA01', false)
			end
			if self.wcAA02 then
				self:SetWeaponEnabledByLabel('EXAA02', true)
				local wepLance02 = self:GetWeaponByLabel('EXAA02')
				wepLance02:ChangeMaxRadius(22)
			else
				self:SetWeaponEnabledByLabel('EXAA02', false)
			end
			if self.wcTMissiles01 then
				self:SetWeaponEnabledByLabel('Missile', true)
				local wepTacMiss = self:GetWeaponByLabel('Missile')
				wepTacMiss:ChangeMaxRadius(256)
			else
				self:SetWeaponEnabledByLabel('Missile', false)
			end
		end
    end,

    OnTransportDetach = function(self, attachBone, unit)
        SWalkingLandUnit.OnTransportDetach(self, attachBone, unit)
		self:StopSiloBuild()
        self:ForkThread(self.WeaponConfigCheck)
    end,

    OnScriptBitClear = function(self, bit)
        if bit == 0 then # shield toggle
			self:DisableShield()
			self:StopUnitAmbientSound( 'ActiveLoop' )
        elseif bit == 8 then # cloak toggle
            self:PlayUnitAmbientSound( 'ActiveLoop' )
            self:SetMaintenanceConsumptionActive()
            self:EnableUnitIntel('Cloak')
            self:EnableUnitIntel('Radar')
            self:EnableUnitIntel('RadarStealth')
            self:EnableUnitIntel('RadarStealthField')
            self:EnableUnitIntel('Sonar')
            self:EnableUnitIntel('SonarStealth')
            self:EnableUnitIntel('SonarStealthField')          
        end
    end,

    OnScriptBitSet = function(self, bit)
        if bit == 0 then # shield toggle
			self:EnableShield()
            self:PlayUnitAmbientSound( 'ActiveLoop' )
		elseif bit == 8 then # cloak toggle
            self:StopUnitAmbientSound( 'ActiveLoop' )
            self:SetMaintenanceConsumptionInactive()
            self:DisableUnitIntel('Cloak')
            self:DisableUnitIntel('Radar')
            self:DisableUnitIntel('RadarStealth')
            self:DisableUnitIntel('RadarStealthField')
            self:DisableUnitIntel('Sonar')
            self:DisableUnitIntel('SonarStealth')
            self:DisableUnitIntel('SonarStealthField')
        end
    end,

    RegenBuffThread = function(self)
        while not self:IsDead() do
            #Get friendly units in the area (including self)
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH1, self:GetPosition(), self:GetBlueprint().Enhancements.EXCombatEngineering.Radius)
            local units2 = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH2, self:GetPosition(), self:GetBlueprint().Enhancements.EXCombatEngineering.Radius)
            local units3 = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH3, self:GetPosition(), self:GetBlueprint().Enhancements.EXCombatEngineering.Radius)
            
            #Give them a 5 second regen buff
            for _,unit in units do
                Buff.ApplyBuff(unit, 'SeraphimACURegenAura')
            end
            for _,unit in units2 do
                Buff.ApplyBuff(unit, 'SeraphimACURegenAura2')
            end
            for _,unit in units3 do
                Buff.ApplyBuff(unit, 'SeraphimACURegenAura3')
            end
            
            #Wait 5 seconds
            WaitSeconds(5)
        end
    end,
       
    AdvancedRegenBuffThread = function(self)
        while not self:IsDead() do
            #Get friendly units in the area (including self)
            local units = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH1, self:GetPosition(), self:GetBlueprint().Enhancements.EXAssaultEngineering.Radius)
            local units2 = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH2, self:GetPosition(), self:GetBlueprint().Enhancements.EXAssaultEngineering.Radius)
            local units3 = AIUtils.GetOwnUnitsAroundPoint(self:GetAIBrain(), categories.TECH3, self:GetPosition(), self:GetBlueprint().Enhancements.EXAssaultEngineering.Radius)
            
            #Give them a 5 second regen buff
            for _,unit in units do
                Buff.ApplyBuff(unit, 'SeraphimAdvancedACURegenAura')
            end
            for _,unit in units2 do
                Buff.ApplyBuff(unit, 'SeraphimAdvancedACURegenAura2')
            end
            for _,unit in units3 do
                Buff.ApplyBuff(unit, 'SeraphimAdvancedACURegenAura3')
            end
            
            #Wait 5 seconds
            WaitSeconds(5)
        end
    end,

    CreateEnhancement = function(self, enh)
        SWalkingLandUnit.CreateEnhancement(self, enh)
        local bp = self:GetBlueprint().Enhancements[enh]
        if not bp then return end
        if enh =='EXImprovedEngineering' then
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT2BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT2BuildRate',
                    DisplayName = 'SERAPHIMACUT2BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT2BuildRate')
            local bp = self:GetBlueprint().Enhancements[enh]
            local bpEcon = self:GetBlueprint().Economy
            if not bp then return end
            self:SetProductionPerSecondEnergy(bp.ProductionPerSecondEnergy + bpEcon.ProductionPerSecondEnergy or 0)
            self:SetProductionPerSecondMass(bp.ProductionPerSecondMass + bpEcon.ProductionPerSecondMass or 0)
            if not Buffs['EXHealthBoost1'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost1',
                    DisplayName = 'EXHealthBoost1',
                    BuffType = 'EXHealthBoost1',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost1')
        elseif enh =='EXImprovedEngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if Buff.HasBuff( self, 'SERAPHIMACUT2BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT2BuildRate' )
            end
            if not bp then return end
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.SERAPHIM * (categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
            local bpEcon = self:GetBlueprint().Economy
            self:SetProductionPerSecondEnergy(bpEcon.ProductionPerSecondEnergy or 0)
            self:SetProductionPerSecondMass(bpEcon.ProductionPerSecondMass or 0)
            if Buff.HasBuff( self, 'EXHealthBoost1' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost1' )
            end
        elseif enh =='EXAdvancedEngineering' then
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT3BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT3BuildRate',
                    DisplayName = 'SERAPHIMCUT3BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT3BuildRate')
            local bp = self:GetBlueprint().Enhancements[enh]
            local bpEcon = self:GetBlueprint().Economy
            if not bp then return end
            self:SetProductionPerSecondEnergy(bp.ProductionPerSecondEnergy + bpEcon.ProductionPerSecondEnergy or 0)
            self:SetProductionPerSecondMass(bp.ProductionPerSecondMass + bpEcon.ProductionPerSecondMass or 0)
            if not Buffs['EXHealthBoost2'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost2',
                    DisplayName = 'EXHealthBoost2',
                    BuffType = 'EXHealthBoost2',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost2')
        elseif enh =='EXAdvancedEngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:RestoreBuildRestrictions()
            if Buff.HasBuff( self, 'SERAPHIMACUT3BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT3BuildRate' )
            end
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
            local bpEcon = self:GetBlueprint().Economy
            self:SetProductionPerSecondEnergy(bpEcon.ProductionPerSecondEnergy or 0)
            self:SetProductionPerSecondMass(bpEcon.ProductionPerSecondMass or 0)
            if Buff.HasBuff( self, 'EXHealthBoost1' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost1' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost2' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost2' )
            end
        elseif enh =='EXExperimentalEngineering' then
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT4BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT4BuildRate',
                    DisplayName = 'SERAPHIMCUT4BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT4BuildRate')
            self:SetRegenRate(bp.NewRegenRate)
            if not Buffs['EXHealthBoost3'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost3',
                    DisplayName = 'EXHealthBoost3',
                    BuffType = 'EXHealthBoost3',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost3')
            if not Buffs['EXRegenBoost1'] then
                BuffBlueprint {
                    Name = 'EXRegenBoost1',
                    DisplayName = 'EXRegenBoost1',
                    BuffType = 'EXRegenBoost1',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        Regen = {
                            Add = bp.NewRegenRate2,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXRegenBoost1')
			elseif enh =='EXExperimentalEngineeringRemove' then
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:RestoreBuildRestrictions()
            if Buff.HasBuff( self, 'SERAPHIMACUT4BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT4BuildRate' )
            end
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
            self:RevertRegenRate()
            if Buff.HasBuff( self, 'EXHealthBoost1' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost1' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost2' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost2' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost3' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost3' )
            end
            if Buff.HasBuff( self, 'EXRegenBoost1' ) then
                Buff.RemoveBuff( self, 'EXRegenBoost1' )
            end
        elseif enh =='EXCombatEngineering' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not Buffs['SeraphimACURegenAura'] then
                BuffBlueprint {
                    Name = 'SeraphimACURegenAura',
                    DisplayName = 'SeraphimACURegenAura',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
                
            end
            if not Buffs['SeraphimACURegenAura2'] then
                BuffBlueprint {
                    Name = 'SeraphimACURegenAura2',
                    DisplayName = 'SeraphimACURegenAura2',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond2 or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
                
            end
            if not Buffs['SeraphimACURegenAura3'] then
                BuffBlueprint {
                    Name = 'SeraphimACURegenAura3',
                    DisplayName = 'SeraphimACURegenAura3',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond3 or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
                
            end
            table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'XSL0001', self:GetArmy(), '/effects/emitters/seraphim_regenerative_aura_01_emit.bp' ) )
            self.RegenThreadHandle = self:ForkThread(self.RegenBuffThread)
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT2BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT2BuildRate',
                    DisplayName = 'SERAPHIMACUT2BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT2BuildRate')
            if not Buffs['EXHealthBoost4'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost4',
                    DisplayName = 'EXHealthBoost4',
                    BuffType = 'EXHealthBoost4',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost4')
            if not Buffs['EXRegenBoost10'] then
                BuffBlueprint {
                    Name = 'EXRegenBoost10',
                    DisplayName = 'EXRegenBoost10',
                    BuffType = 'EXRegenBoost10',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        Regen = {
                            Add = 50,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXRegenBoost10')
        elseif enh =='EXCombatEngineeringRemove' then
            if self.ShieldEffectsBag then
                for k, v in self.ShieldEffectsBag do
                    v:Destroy()
                end
		        self.ShieldEffectsBag = {}
		    end
            KillThread(self.RegenThreadHandle)
            local bp = self:GetBlueprint().Economy.BuildRate
            if Buff.HasBuff( self, 'SERAPHIMACUT2BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT2BuildRate' )
            end
            if not bp then return end
            self:RestoreBuildRestrictions()
            self:AddBuildRestriction( categories.SERAPHIM * (categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
            if Buff.HasBuff( self, 'EXHealthBoost4' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost4' )
            end
            if Buff.HasBuff( self, 'EXRegenBoost10' ) then
                Buff.RemoveBuff( self, 'EXRegenBoost10' )
            end
        elseif enh =='EXAssaultEngineering' then
            if self.RegenThreadHandle then
                if self.ShieldEffectsBag then
                    for k, v in self.ShieldEffectsBag do
                        v:Destroy()
                    end
		            self.ShieldEffectsBag = {}
		        end
                KillThread(self.RegenThreadHandle)
                
            end
            local bp = self:GetBlueprint().Enhancements[enh]
            if not Buffs['SeraphimAdvancedACURegenAura'] then
                BuffBlueprint {
                    Name = 'SeraphimAdvancedACURegenAura',
                    DisplayName = 'SeraphimAdvancedACURegenAura',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
            end
            if not Buffs['SeraphimAdvancedACURegenAura2'] then
                BuffBlueprint {
                    Name = 'SeraphimAdvancedACURegenAura2',
                    DisplayName = 'SeraphimAdvancedACURegenAura2',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond2 or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
            end
            if not Buffs['SeraphimAdvancedACURegenAura3'] then
                BuffBlueprint {
                    Name = 'SeraphimAdvancedACURegenAura3',
                    DisplayName = 'SeraphimAdvancedACURegenAura3',
                    BuffType = 'COMMANDERAURA',
                    Stacks = 'REPLACE',
                    Duration = 5,
                    Affects = {
                        RegenPercent = {
                            Add = 0,
                            Mult = bp.RegenPerSecond3 or 0.1,
                            Ceil = bp.RegenCeiling,
                            Floor = bp.RegenFloor,
                        },
                    },
                }
            end
            table.insert( self.ShieldEffectsBag, CreateAttachedEmitter( self, 'XSL0001', self:GetArmy(), '/effects/emitters/seraphim_regenerative_aura_01_emit.bp' ) )
            self.AdvancedRegenThreadHandle = self:ForkThread(self.AdvancedRegenBuffThread)
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT3BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT3BuildRate',
                    DisplayName = 'SERAPHIMCUT3BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT3BuildRate')
            if not Buffs['EXHealthBoost5'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost5',
                    DisplayName = 'EXHealthBoost5',
                    BuffType = 'EXHealthBoost5',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost5')  
        elseif enh =='EXAssaultEngineeringRemove' then
            if self.ShieldEffectsBag then
                for k, v in self.ShieldEffectsBag do
                    v:Destroy()
                end
		        self.ShieldEffectsBag = {}
		    end
            KillThread(self.AdvancedRegenThreadHandle)
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:RestoreBuildRestrictions()
            if Buff.HasBuff( self, 'SERAPHIMACUT3BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT3BuildRate' )
            end
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )     
            if Buff.HasBuff( self, 'EXHealthBoost4' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost4' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost5' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost5' )
            end
            if Buff.HasBuff( self, 'EXRegenBoost10' ) then
                Buff.RemoveBuff( self, 'EXRegenBoost10' )
            end
        elseif enh =='EXApocolypticEngineering' then
            local cat = ParseEntityCategory(bp.BuildableCategoryAdds)
            self:RemoveBuildRestriction(cat)
            if not Buffs['SERAPHIMACUT4BuildRate'] then
                BuffBlueprint {
                    Name = 'SERAPHIMACUT4BuildRate',
                    DisplayName = 'SERAPHIMCUT4BuildRate',
                    BuffType = 'ACUBUILDRATE',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        BuildRate = {
                            Add =  bp.NewBuildRate - self:GetBlueprint().Economy.BuildRate,
                            Mult = 1,
                        },
                        Regen = {
                            Add = bp.NewRegenRate,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'SERAPHIMACUT4BuildRate')
            self:SetRegenRate(bp.NewRegenRate)
            if not Buffs['EXHealthBoost6'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost6',
                    DisplayName = 'EXHealthBoost6',
                    BuffType = 'EXHealthBoost6',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost6')
            if not Buffs['EXRegenBoost2'] then
                BuffBlueprint {
                    Name = 'EXRegenBoost2',
                    DisplayName = 'EXRegenBoost2',
                    BuffType = 'EXRegenBoost2',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        Regen = {
                            Add = bp.NewRegenRate2,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXRegenBoost2')
        elseif enh =='EXApocolypticEngineeringRemove' then
            if self.ShieldEffectsBag then
                for k, v in self.ShieldEffectsBag do
                    v:Destroy()
                end
		        self.ShieldEffectsBag = {}
		    end
            KillThread(self.AdvancedRegenThreadHandle)
            local bp = self:GetBlueprint().Economy.BuildRate
            if not bp then return end
            self:RestoreBuildRestrictions()
            if Buff.HasBuff( self, 'SERAPHIMACUT4BuildRate' ) then
                Buff.RemoveBuff( self, 'SERAPHIMACUT4BuildRate' )
            end
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER2COMMANDER + categories.BUILTBYTIER3COMMANDER) )
            self:AddBuildRestriction( categories.SERAPHIM * ( categories.BUILTBYTIER4COMMANDER) )
            self:RevertRegenRate()
            if Buff.HasBuff( self, 'EXHealthBoost4' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost4' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost5' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost5' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost6' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost6' )
            end
            if Buff.HasBuff( self, 'EXRegenBoost2' ) then
                Buff.RemoveBuff( self, 'EXRegenBoost2' )
            end
            if Buff.HasBuff( self, 'EXRegenBoost10' ) then
                Buff.RemoveBuff( self, 'EXRegenBoost10' )
            end
		elseif enh =='EXChronotonBooster' then
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            wepChronotron:AddDamageMod(bp.ZephyrDamageMod)        
            wepChronotron:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            wepOvercharge:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
        elseif enh =='EXChronotonBoosterRemove' then
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
        elseif enh =='EXTorpedoLauncher' then
            if not Buffs['EXHealthBoost7'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost7',
                    DisplayName = 'EXHealthBoost7',
                    BuffType = 'EXHealthBoost7',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost7')
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            wepChronotron:AddDamageMod(bp.ZephyrDamageMod)        
            wepChronotron:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            wepOvercharge:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			self.wcTorp01 = true
			self.wcTorp02 = false
			self.wcTorp03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXTorpedoLauncherRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost7' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost7' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcTorp01 = false
			self.wcTorp02 = false
			self.wcTorp03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXTorpedoRapidLoader' then
            if not Buffs['EXHealthBoost8'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost8',
                    DisplayName = 'EXHealthBoost8',
                    BuffType = 'EXHealthBoost8',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost8')
			self.wcTorp01 = false
			self.wcTorp02 = true
			self.wcTorp03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXTorpedoRapidLoaderRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost7' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost7' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost8' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost8' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcTorp01 = false
			self.wcTorp02 = false
			self.wcTorp03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
		elseif enh =='EXTorpedoClusterLauncher' then
            if not Buffs['EXHealthBoost9'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost9',
                    DisplayName = 'EXHealthBoost9',
                    BuffType = 'EXHealthBoost9',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost9')
			self.wcTorp01 = false
			self.wcTorp02 = false
			self.wcTorp03 = true
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXTorpedoClusterLauncherRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost7' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost7' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost8' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost8' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost9' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost9' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcTorp01 = false
			self.wcTorp02 = false
			self.wcTorp03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXCannonBigBall' then
            if not Buffs['EXHealthBoost10'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost10',
                    DisplayName = 'EXHealthBoost10',
                    BuffType = 'EXHealthBoost10',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost10')
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            wepChronotron:AddDamageMod(bp.ZephyrDamageMod)        
            wepChronotron:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            wepOvercharge:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			self.wcBigBall01 = true
			self.wcBigBall02 = false
			self.wcBigBall03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXCannonBigBallRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost10' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost10' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcBigBall01 = false
			self.wcBigBall02 = false
			self.wcBigBall03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXImprovedContainmentBottle' then
            if not Buffs['EXHealthBoost11'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost11',
                    DisplayName = 'EXHealthBoost11',
                    BuffType = 'EXHealthBoost11',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost11')
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            wepChronotron:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			self.wcBigBall01 = false
			self.wcBigBall02 = true
			self.wcBigBall03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXImprovedContainmentBottleRemove' then    
            if Buff.HasBuff( self, 'EXHealthBoost10' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost10' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost11' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost11' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcBigBall01 = false
			self.wcBigBall02 = false
			self.wcBigBall03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXPowerBooster' then
            if not Buffs['EXHealthBoost12'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost12',
                    DisplayName = 'EXHealthBoost12',
                    BuffType = 'EXHealthBoost12',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost12')
            self.wcBigBall01 = false
			self.wcBigBall02 = false
			self.wcBigBall03 = true
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXPowerBoosterRemove' then    
            self:SetWeaponEnabledByLabel('EXBigBallCannon', false)
            if Buff.HasBuff( self, 'EXHealthBoost10' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost10' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost11' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost11' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost12' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost12' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcBigBall01 = false
			self.wcBigBall02 = false
			self.wcBigBall03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXCannonRapid' then
            if not Buffs['EXHealthBoost13'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost13',
                    DisplayName = 'EXHealthBoost13',
                    BuffType = 'EXHealthBoost13',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost13')
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            wepChronotron:AddDamageMod(bp.ZephyrDamageMod)        
            wepChronotron:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            wepOvercharge:ChangeMaxRadius(bp.ZephyrRangeMod or 44)
			self.wcRapid01 = true
			self.wcRapid02 = false
			self.wcRapid03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXCannonRapidRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost13' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost13' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcRapid01 = false
			self.wcRapid02 = false
			self.wcRapid03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXImprovedCoolingSystem' then
            if not Buffs['EXHealthBoost14'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost14',
                    DisplayName = 'EXHealthBoost14',
                    BuffType = 'EXHealthBoost14',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost14')
            self.wcRapid01 = false
			self.wcRapid02 = true
			self.wcRapid03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXImprovedCoolingSystemRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost13' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost13' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost14' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost14' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcRapid01 = false
			self.wcRapid02 = false
			self.wcRapid03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXEnergyShellHardener' then
            if not Buffs['EXHealthBoost15'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost15',
                    DisplayName = 'EXHealthBoost15',
                    BuffType = 'EXHealthBoost15',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost15')
            self.wcRapid01 = false
			self.wcRapid02 = false
			self.wcRapid03 = true
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXEnergyShellHardenerRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost13' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost13' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost14' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost14' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost15' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost15' )
            end
            local wepChronotron = self:GetWeaponByLabel('ChronotronCannon')
            local bpDisruptZephyrRadius = self:GetBlueprint().Weapon[1].MaxRadius
            wepChronotron:ChangeMaxRadius(bpDisruptZephyrRadius or 22)
            wepChronotron:AddDamageMod(-bp.ZephyrDamageMod)
			local wepOvercharge = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOvercharge:ChangeMaxRadius(bpDisruptOCRadius or 22)
			self.wcRapid01 = false
			self.wcRapid02 = false
			self.wcRapid03 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh == 'EXL1Lambda' then
			self.WeaponCheckAA01 = true
			self:SetWeaponEnabledByLabel('EXAA01', true)
			self:SetWeaponEnabledByLabel('EXAA02', true)
			local small01loc = self:GetPosition('S_Lambda_B01')
			local large01loc = self:GetPosition('L_Lambda_B01')
			local lambdaEmitterSm01 = CreateUnit('esb0002', self:GetArmy(), small01loc[1], small01loc[2], small01loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg01 = CreateUnit('esb0001', self:GetArmy(), large01loc[1], large01loc[2], large01loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm01)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg01)
			lambdaEmitterSm01:AttachTo(self, 'S_Lambda_B01')
			lambdaEmitterLg01:AttachTo(self, 'L_Lambda_B01')
			lambdaEmitterSm01:SetParent(self, 'esl0001')
			lambdaEmitterLg01:SetParent(self, 'esl0001')
            lambdaEmitterSm01:SetCreator(self)
            lambdaEmitterLg01:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm01)
			self.Trash:Add(lambdaEmitterLg01)
            if not Buffs['EXHealthBoost22'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost22',
                    DisplayName = 'EXHealthBoost22',
                    BuffType = 'EXHealthBoost22',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost22')
		elseif enh == 'EXL1LambdaRemove' then
			self.WeaponCheckAA01 = false
			self:SetWeaponEnabledByLabel('EXAA01', false)
			self:SetWeaponEnabledByLabel('EXAA02', false)
            if Buff.HasBuff( self, 'EXHealthBoost23' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost23' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost24' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost24' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
        elseif enh == 'EXL2Lambda' then
			local small02loc = self:GetPosition('S_Lambda_B02')
			local large02loc = self:GetPosition('L_Lambda_B02')
			local lambdaEmitterSm02 = CreateUnit('esb0002', self:GetArmy(), small02loc[1], small02loc[2], small02loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg02 = CreateUnit('esb0001', self:GetArmy(), large02loc[1], large02loc[2], large02loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm02)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg02)
			lambdaEmitterSm02:AttachTo(self, 'S_Lambda_B02')
			lambdaEmitterLg02:AttachTo(self, 'L_Lambda_B02')
			lambdaEmitterSm02:SetParent(self, 'esl0001')
			lambdaEmitterLg02:SetParent(self, 'esl0001')
            lambdaEmitterSm02:SetCreator(self)
            lambdaEmitterLg02:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm02)
			self.Trash:Add(lambdaEmitterLg02)
            if not Buffs['EXHealthBoost23'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost23',
                    DisplayName = 'EXHealthBoost23',
                    BuffType = 'EXHealthBoost23',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost23')
        elseif enh == 'EXL2LambdaRemove' then
			self.WeaponCheckAA01 = false
			self:SetWeaponEnabledByLabel('EXAA01', false)
			self:SetWeaponEnabledByLabel('EXAA02', false)
            if Buff.HasBuff( self, 'EXHealthBoost22' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost22' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost23' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost23' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
        elseif enh == 'EXL3Lambda' then
			local small03loc = self:GetPosition('S_Lambda_B03')
			local large03loc = self:GetPosition('L_Lambda_B03')
			local lambdaEmitterSm03 = CreateUnit('esb0002', self:GetArmy(), small03loc[1], small03loc[2], small03loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg03 = CreateUnit('esb0001', self:GetArmy(), large03loc[1], large03loc[2], large03loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm03)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg03)
			lambdaEmitterSm03:AttachTo(self, 'S_Lambda_B03')
			lambdaEmitterLg03:AttachTo(self, 'L_Lambda_B03')
			lambdaEmitterSm03:SetParent(self, 'esl0001')
			lambdaEmitterLg03:SetParent(self, 'esl0001')
            lambdaEmitterSm03:SetCreator(self)
            lambdaEmitterLg03:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm03)
			self.Trash:Add(lambdaEmitterLg03)
            if not Buffs['EXHealthBoost24'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost24',
                    DisplayName = 'EXHealthBoost24',
                    BuffType = 'EXHealthBoost24',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost24')
        elseif enh == 'EXL3LambdaRemove' then
			self.WeaponCheckAA01 = false
			self:SetWeaponEnabledByLabel('EXAA01', false)
			self:SetWeaponEnabledByLabel('EXAA02', false)
            if Buff.HasBuff( self, 'EXHealthBoost22' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost22' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost23' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost23' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost24' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost24' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
        elseif enh == 'EXElectronicsEnhancment' then
			local small01loc = self:GetPosition('S_Lambda_B01')
			local large01loc = self:GetPosition('L_Lambda_B01')
			local lambdaEmitterSm01 = CreateUnit('esb0002', self:GetArmy(), small01loc[1], small01loc[2], small01loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg01 = CreateUnit('esb0001', self:GetArmy(), large01loc[1], large01loc[2], large01loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm01)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg01)
			lambdaEmitterSm01:AttachTo(self, 'S_Lambda_B01')
			lambdaEmitterLg01:AttachTo(self, 'L_Lambda_B01')
			lambdaEmitterSm01:SetParent(self, 'esl0001')
			lambdaEmitterLg01:SetParent(self, 'esl0001')
            lambdaEmitterSm01:SetCreator(self)
            lambdaEmitterLg01:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm01)
			self.Trash:Add(lambdaEmitterLg01)
            self:SetIntelRadius('Vision', bp.NewVisionRadius or 50)
            self:SetIntelRadius('Omni', bp.NewOmniRadius or 50)
            if not Buffs['EXHealthBoost16'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost16',
                    DisplayName = 'EXHealthBoost16',
                    BuffType = 'EXHealthBoost16',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost16')
        elseif enh == 'EXElectronicsEnhancmentRemove' then
            local bpIntel = self:GetBlueprint().Intel
            self:SetIntelRadius('Vision', bpIntel.VisionRadius or 26)
            self:SetIntelRadius('Omni', bpIntel.OmniRadius or 26)
            if Buff.HasBuff( self, 'EXHealthBoost16' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost16' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
        elseif enh == 'EXElectronicCountermeasures' then
			local small02loc = self:GetPosition('S_Lambda_B02')
			local large02loc = self:GetPosition('L_Lambda_B02')
			local lambdaEmitterSm02 = CreateUnit('esb0002', self:GetArmy(), small02loc[1], small02loc[2], small02loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg02 = CreateUnit('esb0001', self:GetArmy(), large02loc[1], large02loc[2], large02loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm02)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg02)
			lambdaEmitterSm02:AttachTo(self, 'S_Lambda_B02')
			lambdaEmitterLg02:AttachTo(self, 'L_Lambda_B02')
			lambdaEmitterSm02:SetParent(self, 'esl0001')
			lambdaEmitterLg02:SetParent(self, 'esl0001')
            lambdaEmitterSm02:SetCreator(self)
            lambdaEmitterLg02:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm02)
			self.Trash:Add(lambdaEmitterLg02)
            self:AddToggleCap('RULEUTC_CloakToggle')
            if self.IntelEffectsBag then
                EffectUtil.CleanupEffectBag(self,'IntelEffectsBag')
                self.IntelEffectsBag = nil
            end
            self.CloakEnh = false        
            self.StealthEnh = true
            self:EnableUnitIntel('RadarStealth')
            self:EnableUnitIntel('SonarStealth')
            self:AddCommandCap('RULEUCC_Teleport')
            if not Buffs['EXHealthBoost17'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost17',
                    DisplayName = 'EXHealthBoost17',
                    BuffType = 'EXHealthBoost17',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost17')
			self.wcAA01 = true
			self.wcAA02 = true
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh == 'EXElectronicCountermeasuresRemove' then
            self:RemoveToggleCap('RULEUTC_CloakToggle')
            self:DisableUnitIntel('RadarStealth')
            self:DisableUnitIntel('SonarStealth')           
            self.StealthEnh = false
            self.CloakEnh = false 
            self.StealthFieldEffects = false
            self.CloakingEffects = false     
            self:RemoveCommandCap('RULEUCC_Teleport')
            local bpIntel = self:GetBlueprint().Intel
            self:SetIntelRadius('Vision', bpIntel.VisionRadius or 26)
            self:SetIntelRadius('Omni', bpIntel.OmniRadius or 26)
            if Buff.HasBuff( self, 'EXHealthBoost16' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost16' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost17' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost17' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
			self.wcAA01 = false
			self.wcAA02 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh == 'EXCloakingSubsystems' then
            local bp = self:GetBlueprint().Enhancements[enh]
            if not bp then return end
            self.StealthEnh = false
			self.CloakEnh = true 
            self:EnableUnitIntel('Cloak')
            if not Buffs['EXHealthBoost18'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost18',
                    DisplayName = 'EXHealthBoost18',
                    BuffType = 'EXHealthBoost18',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost18')
        elseif enh == 'EXCloakingSubsystemsRemove' then
            self:RemoveToggleCap('RULEUTC_CloakToggle')
            self:DisableUnitIntel('Cloak')
            self.CloakEnh = false 
            self:RemoveCommandCap('RULEUCC_Teleport')
            local bpIntel = self:GetBlueprint().Intel
            self:SetIntelRadius('Vision', bpIntel.VisionRadius or 26)
            self:SetIntelRadius('Omni', bpIntel.OmniRadius or 26)
            if Buff.HasBuff( self, 'EXHealthBoost16' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost16' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost17' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost17' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost18' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost18' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
			self.wcAA01 = false
			self.wcAA02 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXBasicDefence' then
			local small01loc = self:GetPosition('S_Lambda_B01')
			local large01loc = self:GetPosition('L_Lambda_B01')
			local lambdaEmitterSm01 = CreateUnit('esb0002', self:GetArmy(), small01loc[1], small01loc[2], small01loc[3], 0, 0, 0, 0, 'Land')
			local lambdaEmitterLg01 = CreateUnit('esb0001', self:GetArmy(), large01loc[1], large01loc[2], large01loc[3], 0, 0, 0, 0, 'Land')
			table.insert (self.lambdaEmitterTable, lambdaEmitterSm01)
			table.insert (self.lambdaEmitterTable, lambdaEmitterLg01)
			lambdaEmitterSm01:AttachTo(self, 'S_Lambda_B01')
			lambdaEmitterLg01:AttachTo(self, 'L_Lambda_B01')
			lambdaEmitterSm01:SetParent(self, 'esl0001')
			lambdaEmitterLg01:SetParent(self, 'esl0001')
            lambdaEmitterSm01:SetCreator(self)
            lambdaEmitterLg01:SetCreator(self)
			self.Trash:Add(lambdaEmitterSm01)
			self.Trash:Add(lambdaEmitterLg01)
            if not Buffs['EXHealthBoost19'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost19',
                    DisplayName = 'EXHealthBoost19',
                    BuffType = 'EXHealthBoost19',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost19')
			local wepOC = self:GetWeaponByLabel('OverCharge')
            wepOC:ChangeMaxRadius(bp.OverchargeRangeMod or 44)
            wepOC:AddDamageMod(bp.OverchargeDamageMod)        
        elseif enh =='EXBasicDefenceRemove' then
            if Buff.HasBuff( self, 'EXHealthBoost19' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost19' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
			local wepOC = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOC:ChangeMaxRadius(bpDisruptOCRadius or 22)
            wepOC:AddDamageMod(-bp.OverchargeDamageMod)        
			self:StopSiloBuild()
        elseif enh =='EXTacticalMisslePack' then
            self:AddCommandCap('RULEUCC_Tactical')
            self:AddCommandCap('RULEUCC_SiloBuildTactical')
            if not Buffs['EXHealthBoost20'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost20',
                    DisplayName = 'EXHealthBoost20',
                    BuffType = 'EXHealthBoost20',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost20')
			local wepOC = self:GetWeaponByLabel('OverCharge')
            wepOC:AddDamageMod(bp.OverchargeDamageMod2)        
			self.wcTMissiles01 = true
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh == 'EXTacticalMisslePackRemove' then
            self:RemoveCommandCap('RULEUCC_Tactical')
            self:RemoveCommandCap('RULEUCC_SiloBuildTactical')
            local amt = self:GetTacticalSiloAmmoCount()
            self:RemoveTacticalSiloAmmo(amt or 0)
            self:StopSiloBuild()
            if Buff.HasBuff( self, 'EXHealthBoost19' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost19' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost20' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost20' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost21' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost21' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
			local wepOC = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOC:ChangeMaxRadius(bpDisruptOCRadius or 22)
            wepOC:AddDamageMod(-bp.OverchargeDamageMod)        
            wepOC:AddDamageMod(-bp.OverchargeDamageMod2)        
			self.wcTMissiles01 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
        elseif enh =='EXOverchargeOverdrive' then
            if not Buffs['EXHealthBoost21'] then
                BuffBlueprint {
                    Name = 'EXHealthBoost21',
                    DisplayName = 'EXHealthBoost21',
                    BuffType = 'EXHealthBoost21',
                    Stacks = 'REPLACE',
                    Duration = -1,
                    Affects = {
                        MaxHealth = {
                            Add = bp.NewHealth,
                            Mult = 1.0,
                        },
                    },
                }
            end
            Buff.ApplyBuff(self, 'EXHealthBoost21')   
			local wepOC = self:GetWeaponByLabel('OverCharge')
            wepOC:ChangeMaxRadius(bp.OverchargeRangeMod or 44)
            wepOC:AddDamageMod(bp.OverchargeDamageMod3)        
			wepOC:ChangeProjectileBlueprint(bp.NewProjectileBlueprint)
        elseif enh == 'EXOverchargeOverdriveRemove' then
            self:RemoveCommandCap('RULEUCC_Tactical')
            self:RemoveCommandCap('RULEUCC_SiloBuildTactical')
            local amt = self:GetTacticalSiloAmmoCount()
            self:RemoveTacticalSiloAmmo(amt or 0)
            self:StopSiloBuild()
            if Buff.HasBuff( self, 'EXHealthBoost19' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost19' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost20' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost20' )
            end
            if Buff.HasBuff( self, 'EXHealthBoost21' ) then
                Buff.RemoveBuff( self, 'EXHealthBoost21' )
            end
			if table.getn({self.lambdaEmitterTable}) > 0 then
				for k, v in self.lambdaEmitterTable do 
					IssueClearCommands({self.lambdaEmitterTable[k]}) 
					IssueKillSelf({self.lambdaEmitterTable[k]})
				end
			end
			local wepOC = self:GetWeaponByLabel('OverCharge')
            local bpDisruptOCRadius = self:GetBlueprint().Weapon[2].MaxRadius
            wepOC:ChangeMaxRadius(bpDisruptOCRadius or 22)
            wepOC:AddDamageMod(-bp.OverchargeDamageMod)        
            wepOC:AddDamageMod(-bp.OverchargeDamageMod2)        
            wepOC:AddDamageMod(-bp.OverchargeDamageMod3)        
			wepOC:ChangeProjectileBlueprint(bp.NewProjectileBlueprint)
			self.wcTMissiles01 = false
			self:ForkThread(self.WeaponRangeReset)
			self:ForkThread(self.WeaponConfigCheck)
		end
    end,

    IntelEffects = {
		Cloak = {
		    {
			    Bones = {
				    'Body',
--				    'Right_Turret',
--				    'Left_Turret',
				    'Right_Arm_B01',
				    'Left_Arm_B01',
				    'Torso',
--				    'Chest_Left',
				    'Left_Leg_B01',
				    'Left_Leg_B02',
				    'Right_Leg_B01',
				    'Right_Leg_B02',
			    },
			    Scale = 1.0,
			    Type = 'Cloak01',
		    },
		},
		Field = {
		    {
			    Bones = {
				    'Body',
--				    'Right_Turret',
--				    'Left_Turret',
				    'Right_Arm_B01',
				    'Left_Arm_B01',
				    'Torso',
--				    'Chest_Left',
				    'Left_Leg_B01',
				    'Left_Leg_B02',
				    'Right_Leg_B01',
				    'Right_Leg_B02',
			    },
			    Scale = 1.6,
			    Type = 'Cloak01',
		    },	
        },	
    },
	
    OnIntelEnabled = function(self)
        SWalkingLandUnit.OnIntelEnabled(self)
        if self.CloakEnh and self:IsIntelEnabled('Cloak') then 
            self:SetEnergyMaintenanceConsumptionOverride(self:GetBlueprint().Enhancements['EXCloakingSubsystems'].MaintenanceConsumptionPerSecondEnergy or 0)
            self:SetMaintenanceConsumptionActive()
            if not self.IntelEffectsBag then
			    self.IntelEffectsBag = {}
			    self.CreateTerrainTypeEffects( self, self.IntelEffects.Cloak, 'FXIdle',  self:GetCurrentLayer(), nil, self.IntelEffectsBag )
			end            
        elseif self.StealthEnh and self:IsIntelEnabled('RadarStealth') and self:IsIntelEnabled('SonarStealth') then
            self:SetEnergyMaintenanceConsumptionOverride(self:GetBlueprint().Enhancements['EXElectronicCountermeasures'].MaintenanceConsumptionPerSecondEnergy or 0)
            self:SetMaintenanceConsumptionActive()  
            if not self.IntelEffectsBag then 
	            self.IntelEffectsBag = {}
		        self.CreateTerrainTypeEffects( self, self.IntelEffects.Field, 'FXIdle',  self:GetCurrentLayer(), nil, self.IntelEffectsBag )
		    end                  
        end		
    end,

    OnIntelDisabled = function(self)
        SWalkingLandUnit.OnIntelDisabled(self)
        if self.IntelEffectsBag then
            EffectUtil.CleanupEffectBag(self,'IntelEffectsBag')
            self.IntelEffectsBag = nil
        end
        if self.CloakEnh and not self:IsIntelEnabled('Cloak') then
            self:SetMaintenanceConsumptionInactive()
        elseif self.StealthEnh and not self:IsIntelEnabled('RadarStealth') and not self:IsIntelEnabled('SonarStealth') then
            self:SetMaintenanceConsumptionInactive()
        end         
    end,

    OnPaused = function(self)
        SWalkingLandUnit.OnPaused(self)
        if self.BuildingUnit then
            SWalkingLandUnit.StopBuildingEffects(self, self:GetUnitBeingBuilt())
        end
    end,

    OnUnpaused = function(self)
        if self.BuildingUnit then
            SWalkingLandUnit.StartBuildingEffects(self, self:GetUnitBeingBuilt(), self.UnitBuildOrder)
        end
        SWalkingLandUnit.OnUnpaused(self)
    end,

}

TypeClass = ESL0001